from funasr import AutoModel
from typing import List, Dict, Any
from src.models import VADResponse
from src.models import AudioBinary_Config
from src.models import AudioBinary_data_list
from src.models import AudioBinary_Slice
from typing import Callable
from src.functor.base import BaseFunctor
import threading
from queue import Empty, Queue

# 日志
from src.utils.logger import get_module_logger

logger = get_module_logger(__name__)


class VADFunctor(BaseFunctor):
    def __init__(
        self
    ):
        super().__init__()
        self._model: dict = {}
        self._callback: List[Callable] = []
        self._status_lock: threading.Lock = threading.Lock()
        self._input_queue: Queue = None
        self._audio_config: AudioBinary_Config = None
        self._is_running: bool = False
        self._stop_event: bool = False
        self._audio_cache: bytes = b''
        self._cache: dict = {}

    def set_input_queue(self, queue: Queue):
        self._input_queue = queue

    def set_model(self, model: dict):
        self._model = model

    def set_audio_config(self, audio_config: AudioBinary_Config):
        self._audio_config = audio_config

    def add_callback(self, callback: Callable):
        if not isinstance(self._callback, list):
            self._callback = []
        self._callback.append(callback)

    def _process(self, data: bytes):
        """
        处理数据
        """
        self._audio_cache += data
        if len(self._audio_cache) >= self._audio_config.chunk_size*100:
            result = self._model['vad'].generate(
                input=self._audio_cache,
                cache=self._cache,
                chunk_size=self._audio_config.chunk_size,
                is_final=False,
            )
            logger.info(f"VADFunctor处理数据: {len(self._audio_cache)}, {result}")
            self._audio_cache = b''
        

    def _run(self):
        """
        线程运行逻辑
        监听输入队列，当有数据时，处理数据
        当输入队列为空时, 间隔1s检测是否进入停止事件。
        """
        # 刷新运行状态
        with self._status_lock:
            self._is_running = True
            self._stop_event = False
        # 运行逻辑
        while self._is_running:
            try:
                data = self._input_queue.get(True, timeout=1)
                self._process(data)
                self._input_queue.task_done()
            # 当队列为空时，间隔1s检测是否进入停止事件。
            except Empty:
                if self._stop_event:
                    break
                continue
            # 其他异常
            except Exception as e:
                logger.error(f"VADFunctor运行时发生错误: {e}")
                raise e

    def run(self):
        """
        启动 _run 线程, 并返回线程对象
        """
        self._thread = threading.Thread(target=self._run, daemon=True)
        self._thread.start()
        return self._thread

    def _pre_check(self):
        pass

    def stop(self):
        with self._status_lock:
            self._stop_event = True
        self._thread.join()
        with self._status_lock:
            self._is_running = False
        return True


# class VAD:

#     def __init__(
#         self,
#         VAD_model=None,
#         audio_config: AudioBinary_Config = None,
#         callback: Callable = None,
#     ):
#         # vad model
#         self.VAD_model = VAD_model
#         if self.VAD_model is None:
#             self.VAD_model = AutoModel(
#                 model="fsmn-vad", model_revision="v2.0.4", disable_update=True
#             )
#         # audio config
#         self.audio_config = audio_config
#         # vad result
#         self.vad_result = VADResponse(time_chunk_index_callback=callback)
#         # audio binary poll
#         self.audio_chunk = AudioChunk(audio_config=self.audio_config)
#         self.cache = {}

#     def push_binary_data(
#         self,
#         binary_data: bytes,
#     ):
#         # 压入二进制数据
#         self.audio_chunk.add_chunk(binary_data)
#         # 处理音频块
#         res = self.VAD_model.generate(
#             input=binary_data,
#             cache=self.cache,
#             chunk_size=self.audio_config.chunk_size,
#             is_final=False,
#         )
#         # print("VAD generate", res)
#         if len(res[0]["value"]):
#             self.vad_result += VADResponse.from_raw(res)

#     def set_callback(
#         self,
#         callback: Callable,
#     ):
#         self.vad_result.time_chunk_index_callback = callback

#     def process_vad_result(self, callback: Callable = None):
#         # 处理VAD结果
#         callback = (
#             callback
#             if callback is not None
#             else self.vad_result.time_chunk_index_callback
#         )
#         self.vad_result.process_time_chunk(
#             lambda x: callback(
#                 AudioBinary_Chunk(
#                     start_time=x["start_time"],
#                     end_time=x["end_time"],
#                     chunk=self.audio_chunk.get_chunk(x["start_time"], x["end_time"]),
#                 )
#             )
#         )
